#!/usr/bin/python
# -*- coding: utf-8 

# System
import sys
import statistics
import math

from datetime import time, datetime, date, timedelta

# Utility
import pandas
import seaborn

from pandas.tools import plotting
import matplotlib.pyplot as plt

# Custom - Enliten
import common.ibert_common as ic
import common.query_strings as qs
import common.enliten_env_stat as ev
import common.enliten_energy_stat as ee

'''
definitions

'''

dataModelByType = {'home':[], 'type':[], 'before':[], 'after':[], 
			'wb_nvna':[], 'nvna':[], 'wb_vna':[], 'vna':[], 
			'wb_nva':[], 'nva':[], 'wb_va':[], 'va':[]}


# Create the data for generating CSV
def setDataByType(homeid, items, dataModelByType):
	# filter out items that have a different home id 
	# only a set of items having the same home id will be used
	validItems = filter(lambda i: i['home'] is homeid, items)
	for vitem in validItems:
		dataModelByType['home'].append(vitem['home'])
		dataModelByType['type'].append(vitem['type'])
		for qitem in ic.quadrant:
			dataModelByType[qitem].append(vitem[qitem])

#
def analyseQuadrantFull():

	#Reading CSV
	full = pandas.read_csv('./data/ibert_study_daily_full.csv', sep=',', na_values='NaN', parse_dates=['timestamp'])

	# get full home list
	homes = ic.getEnlitenHomes()
	items = []

	start = ic.PHASE_ZERO['from']
	end = ic.PHASE_FIVE['to']

	for home in homes:
	
		print "[home %d] ----------" % home['home']
	
		items.append(ev.getQuadrantStatsFullFromDF(home, qs.TYPE_ENV_T, start, end, full))
		print "Temperature done!"

		items.append(ev.getQuadrantStatsFullFromDF(home, qs.TYPE_ENV_C, start, end, full))
		print "CO2 done!"

		items.append(ev.getQuadrantStatsFullFromDF(home, qs.TYPE_ENV_E, start, end, full))
		print "Electricity done!"

		items.append(ev.getQuadrantStatsFullFromDF(home, qs.TYPE_ENV_G,  start, end, full))
		print "Gas_spend done!"

		setDataByType(home['home'], items, dataModelByType)

'''
Program Starts!
'''

# init
plt.rcdefaults()

# getData
analyseQuadrantFull()

# save to csv
converted = pandas.DataFrame(dataModelByType)

colorder = ['home', 'type']
colorder.extend(ic.quadrant)
converted = converted[colorder]

converted.to_csv('./data/ibert_study_QuadStat_full.csv', sep=',', nanRep='NaN')

print "end..."

	 
