#!/usr/bin/python
# -*- coding: utf-8 

# systems
import sys
import statistics
import numpy as np

from datetime import time, datetime, date, timedelta

# customs
import mysql_util as dbutil	
import query_strings as qs		
import ibert_common as ic

'''
Functions
'''
#
#
#
def setItemOld(outItem, phase, group, mean, stdev):
	if phase < 0:
		print 'phase: out of range (which should be 0 <= phase < =5'
		return
	
	if phase < 1 or phase > 4:
		outItem[ic.quadrant[phase]] = mean
	else:
		idx = (phase + group - 1) % 4
		qid = idx if idx > 0 else 4
		outItem[ic.quadrant[qid]] = mean

#
def setItem(outItem, phase, group, mean, stdev):
	if phase < 0:
		print 'phase: out of range (which should be 0 <= phase < =5'
		return
	
	if phase < 1 or phase > 8:
		outItem[ic.quadrant[phase]] = mean
	else:
		idx = ((phase/2 + phase%2) + group - 1) % 4 
		idx = 4 if idx is 0 else idx 
		qid = idx * 2 - phase % 2
		outItem[ic.quadrant[qid]] = mean

def QSTART(datefrom, phase):
	start = datefrom
	offset = timedelta(days = 14)
	
	if phase > 0 and phase < 9 and (phase % 2) is 1:
		start += offset
		
	return start
	
def QEND(dateto, phase):
	end = dateto
	offset = timedelta(days = 14)
	
	if phase > 0 and phase < 9 and (phase % 2) is 0:
		end -= offset
		
	return end

# weekly 
# this may absorb the quadrant stat api below with the option argument
def getQuadrantStatsWeeklyBasis(home, energyType, outList):
	item = {'home':'%s', 'type':'%s', 'before':'%d', 'after':'%d', 
			'wb_nvna':'%d', 'nvna':'%d', 'wb_vna':'%d', 'vna':'%d', 
			'wb_nva':'%d', 'nva':'%d', 'wb_va':'%d', 'va':'%d'}

	item['home'] = home['home']
	item['type'] = energyType

	start = ic.PHASE_ZERO['from']
	end = ic.PHASE_FIVE['to']
	records = dbutil.select(qs.ENERGY % (int(home["home"]), start, end))

	dateStart = datetime.strptime(start, '%Y-%m-%d %H:%M:%S').date()
	dateEnd = datetime.strptime(end, '%Y-%m-%d %H:%M:%S').date()
	datePhaseFrom = dateStart
	datePhaseTo = datePhaseFrom + timedelta(days = 21)
	
	phase = 0
	group = int(home['schedule_group'])

	while (datePhaseTo <= dateEnd):
		phaseList = filter(lambda r : 
						r['timestamp'].date() >= QSTART(datePhaseFrom, phase) and 
						r['timestamp'].date() < QEND(datePhaseTo, phase) and
						r[energyType] is not None and
						r[energyType] > 0,
						records)

		phaseTempList = [t[energyType] for t in phaseList]

		mean = statistics.mean(phaseTempList) if len(phaseTempList) > 0 else float('nan')	
		stdev = statistics.stdev(phaseTempList) if len(phaseTempList) > 1 else 0
		setItem(item, phase, group, mean, stdev)

		phase += 1
		if (phase > 0 and (phase % 2) is 0) or (phase is 9): 
			datePhaseFrom = datePhaseTo
			datePhaseTo += timedelta(days = 21)

	print item
	outList.append(item)
	return item

# returns one home's energy (gas/elec) statistics in the quadrant basis
# in : the whole temperature data from phase 0 to phase 5
# in : home id
# 
# This should be unified with the above api
def getQuadrantStatistics(home, energyType, outList):
	item = {'home':'%s', 'type':'%s', 'before':'%d', 'after':'%d', 
			'wb_nvna':'%d', 'nvna':'%d', 'wb_vna':'%d', 'vna':'%d', 
			'wb_nva':'%d', 'nva':'%d', 'wb_va':'%d', 'va':'%d'}

	item['home'] = home['home']
	item['type'] = energyType

	start = ic.PHASE_ZERO['from']
	end = ic.PHASE_FIVE['to']
	records = dbutil.select(qs.ENERGY % (int(home["home"]), start, end))

	dateStart = datetime.strptime(start, '%Y-%m-%d %H:%M:%S').date()
	dateEnd = datetime.strptime(end, '%Y-%m-%d %H:%M:%S').date()
	datePhaseFrom = dateStart
	datePhaseTo = datePhaseFrom + timedelta(days = 21)
	
	phase = 0
	group = int(home['schedule_group'])

	while (datePhaseTo <= dateEnd):
		phaseList = filter(lambda r : 
						r['timestamp'].date() >= datePhaseFrom and 
						r['timestamp'].date() < datePhaseTo and
						r[energyType] is not None and
						r[energyType] > 0,
						records)

		phaseTempList = [t[energyType] for t in phaseList]

		mean = statistics.mean(phaseTempList) if len(phaseTempList) > 0 else float('nan')	
		stdev = statistics.stdev(phaseTempList) if len(phaseTempList) > 1 else 0
		setItem(item, phase, group, mean, stdev)

		phase += 1
		datePhaseFrom = datePhaseTo
		datePhaseTo += timedelta(days = 21)

	print item
	outList.append(item)
	return item

# Daily
def getDailyStats(home, energyType, envDataList):

	means = []
	
	start = ic.PHASE_ZERO['from']
	end = ic.PHASE_FIVE['to']
	records = dbutil.select(qs.ENERGY % (int(home['home']), start, end))

	dateStart = datetime.strptime(start, '%Y-%m-%d %H:%M:%S').date()
	dateEnd = datetime.strptime(end, '%Y-%m-%d %H:%M:%S').date()
	dateCur = dateStart
	
	while (dateCur <= dateEnd):
		phaseList = filter(lambda r : 
						r['timestamp'].date() == dateCur and 
						r[energyType] is not None and
						r[energyType] > 0,
						records)
		values = [t[energyType] for t in phaseList]
		
		mean = statistics.mean(values) if len(values) > 0 else float('nan')
		if (len(values) > 1): stdev = statistics.stdev(values)
		elif (len(values) == 1): stdev = 0;
		else: stdev = float('nan')
		
		envDataList['home'].append(home['home'])
		envDataList['type'].append(energyType)		
		envDataList['value'].append(mean)
		envDataList['stdev'].append(stdev)
		envDataList['timestamp'].append(dateCur)
		envDataList['room'].append(float('nan'))		
		
		means.append(mean)
		dateCur += timedelta(days = 1)
	
	return len(means)
