#!/usr/bin/python
# -*- coding: utf-8 

# systems
import sys
import statistics	

from datetime import time, datetime, date, timedelta

# customs
import mysql_util as dbutil	
import query_strings as qs		
import ibert_common as ic

'''
Dictionary --

1. Study related terms
	nvna	no-value/no-action-prompt
	vna		value/no-action-prompt
	nva		no-value/action-prompt
	va		value/action-prompt	

2. Pre- and Post-fix
	wb_		week before
	w_		week
	wa_		week after
	
3. Mics
	type	type of sensor reading e.g. temperature, co2, humidity, 
			ibert playing time, electricity spend, gas spend etc
	phase	a period that receives a particular type of feedback (see 1.)
'''

'''
Functions
'''

def getQuadrantWeeklyRecords(home, envtype, outList):
	witem = {'home':'%s', 'type':'%s', 
			 'before':'%d', 'after':'%d', 
			'nvnp':'%d', 'vnp':'%d', 'nvp':'%d', 'vp':'%d'}
	
def setItem(outItem, phase, group, mean, stdev):
	if phase < 0:
		print 'phase: out of range (which should be 0 <= phase < =5'
		return
	
	if phase < 1 or phase > 8:
		if (phase > 8):
			phase = 9
		outItem[ic.quadrant[phase]] = mean
	else:
		idx = ((phase/2 + phase%2) + group - 1) % 4 
		idx = 4 if idx is 0 else idx 
		qid = idx * 2 - phase % 2
		outItem[ic.quadrant[qid]] = mean
		
def QSTART(datefrom, phase):
	start = datefrom
	offset = timedelta(days = 14)
	
	if phase > 0 and phase < 9 and (phase % 2) is 1:
		start += offset
		
	return start
	
def QEND(dateto, phase):
	end = dateto
	offset = timedelta(days = 14)
	
	if phase > 0 and phase < 9 and (phase % 2) is 0:
		end -= offset
		
	return end

'''
API - getQuadrantStatsWeeklyBasisFromDB()

@Descrition: 
	- Retrieve statistics for each quadrant, 
	- but particularly looking at a week before and after the feedback released to users
	- get statistics from main DB
	- It is able to cover for both environmental and energy data

@Arguments
	- home		input, object, includes home id and schedule group
	- envType	input, string, a type of env/energy such as temperature, co2, electricity etc
				see ./common/ibert_common.py for more options
	- start		input, start date for the analysis
	- end		input, end date for the analysis
	- outList	output, output buffer for the result
'''
	
def getQuadrantStatsWeeklyBasisFromDB(home, envType, start, end, outList):
	item = {'home':'%s', 'type':'%s', 'before':'%d', 'after':'%d', 
			'wb_nvna':'%d', 'nvna':'%d', 'wb_vna':'%d', 'vna':'%d', 
			'wb_nva':'%d', 'nva':'%d', 'wb_va':'%d', 'va':'%d'}
	
	item['home'] = home['home']
	item['type'] = envType
	group = int(home['schedule_group'])
	phase = 0
	
	start = ic.PHASE_ZERO['from']
	end = ic.PHASE_FIVE['to']
	records = dbutil.select(qs.ENV_QUERY_MAP[envType] % (int(home["home"]), start, end))
	
	dateStart = datetime.strptime(start, '%Y-%m-%d %H:%M:%S').date()
	dateEnd = datetime.strptime(end, '%Y-%m-%d %H:%M:%S').date()
	datePhaseFrom = dateStart
	datePhaseTo = datePhaseFrom + timedelta(days = 21)

	while (datePhaseTo <= dateEnd):
		phaseList = filter(lambda r : 
						r['timestamp'].date() >= QSTART(datePhaseFrom, phase) and 
						r['timestamp'].date() < QEND(datePhaseTo, phase),
						records)
		phaseTempList = [t[qs.ENV_COL_MAP[envType]] for t in phaseList]

		mean = statistics.mean(phaseTempList) if len(phaseTempList) > 0 else float('nan')	
		stdev = statistics.stdev(phaseTempList) if len(phaseTempList) > 1 else float('nan')
		setItem(item, phase, group, mean, stdev)
		
		phase += 1
		if (phase > 0 and (phase % 2) is 0) or (phase is 9): 
			datePhaseFrom = datePhaseTo
			datePhaseTo += timedelta(days = 21)

	print item
	outList.append(item)
	return item

'''
API - getQuadStatsWeeklyBasisFromDF()

@Descrition: 
	- Retrieve statistics for each quadrant, 
	- but particularly looking at a week before and after the feedback released to users
	- get statistics from pandas dataframe
	- It is able to cover for both environmental and energy data

@Arguments
	- home		input, object, includes home id and schedule group
	- envType	input, string, a type of env/energy such as temperature, co2, electricity etc
				see ./common/ibert_common.py for more options
	- start		start date for the analysis
	- end		end date for the analysis
	- dataframe	a full data for 18 weeks for all homes
'''
def getQuadStatsWeeklyBasisFromDF(home, envType, start, end, dataframe):
	#item = ic.DATA_ITEM
	item = {'home':'%s', 'type':'%s', 'before':'%d', 'after':'%d', 
			'wb_nvna':'%d', 'nvna':'%d', 'wb_vna':'%d', 'vna':'%d', 
			'wb_nva':'%d', 'nva':'%d', 'wb_va':'%d', 'va':'%d'}
	
	item['home'] = home['home']
	item['type'] = envType
	group = int(home['schedule_group'])
	phase = 0
	
	records = dataframe[dataframe['home'] == home['home']]
	records = records[records['type'] == envType]
	
	dateStart = datetime.strptime(start, '%Y-%m-%d %H:%M:%S').date()
	dateEnd = datetime.strptime(end, '%Y-%m-%d %H:%M:%S').date()
	datePhaseFrom = dateStart
	datePhaseTo = datePhaseFrom + timedelta(days = 21)

	while (datePhaseTo <= dateEnd):
		phaseDF = records		
		phaseDF = phaseDF[phaseDF['timestamp'] >= QSTART(datePhaseFrom, phase)]
		phaseDF = phaseDF[phaseDF['timestamp'] < QEND(datePhaseTo, phase)]

		mean = phaseDF['value'].mean()
		stdev = phaseDF['value'].std()
		setItem(item, phase, group, mean, stdev)
		
		phase += 1
		if (phase > 0 and (phase % 2) is 0) or (phase is 9): 
			datePhaseFrom = datePhaseTo
			datePhaseTo += timedelta(days = 21)

	print item
	return item
		
# returns one home's environment statistics in the quadrant basis
# in : the whole environmental data from phase 0 to phase 5
# in : a type of environment data (temprature / 
# in : home id
# 
# should be revised
def getQuadrantStatistics(home, envType, outList):
	item = {'home':'%s', 'type':'%s', 'before':'%d', 'after':'%d', 
			'nvnp':'%d', 'vnp':'%d', 'nvp':'%d', 'vp':'%d'}	

	item['home'] = home['home']
	item['type'] = envType

	start = ic.PHASE_ZERO['from']
	end = ic.PHASE_FIVE['to']
	records = dbutil.select(qs.ENV_QUERY_MAP[envType] % (int(home["home"]), start, end))

	dateStart = datetime.strptime(start, '%Y-%m-%d %H:%M:%S').date()
	dateEnd = datetime.strptime(end, '%Y-%m-%d %H:%M:%S').date()
	datePhaseFrom = dateStart
	datePhaseTo = datePhaseFrom + timedelta(days = 21)
	
	phase = 0
	group = int(home['schedule_group'])

	while (datePhaseTo <= dateEnd):
		phaseList = filter(lambda r : 
						r['timestamp'].date() >= datePhaseFrom and 
						r['timestamp'].date() < datePhaseTo,
						records)
		phaseTempList = [t[qs.ENV_COL_MAP[envType]] for t in phaseList]

		mean = statistics.mean(phaseTempList) if len(phaseTempList) > 0 else float('nan')	
		stdev = statistics.stdev(phaseTempList) if len(phaseTempList) > 1 else float('nan')
		setItem(item, phase, group, mean, stdev)

		phase += 2
		datePhaseFrom = datePhaseTo
		datePhaseTo += timedelta(days = 21)

	print item
	outList.append(item)
	return item

# returns one home's environment statistics in the quadrant basis
# in : the whole environmental data from phase 0 to phase 5
# in : a type of environment data (temprature / 
# in : home id
# 
# should be revised
def getQuadrantStatsFullFromDF(home, envType, start, end, dataframe):
	item = {'home':'%s', 'type':'%s', 'before':'%d', 'after':'%d', 
			'wb_nvna':float('nan'), 'nvna':'%d', 'wb_vna':float('nan'), 'vna':'%d', 
			'wb_nva':float('nan'), 'nva':'%d', 'wb_va':float('nan'), 'va':'%d'}

	item['home'] = home['home']
	item['type'] = envType
	phase = 0
	group = int(home['schedule_group'])

	records = dataframe[dataframe['home'] == home['home']]
	records = records[records['type'] == envType]

	dateStart = datetime.strptime(start, '%Y-%m-%d %H:%M:%S').date()
	dateEnd = datetime.strptime(end, '%Y-%m-%d %H:%M:%S').date()
	datePhaseFrom = dateStart
	datePhaseTo = datePhaseFrom + timedelta(days = 21)
	
	while (datePhaseTo <= dateEnd):
		phaseDF = records		
		phaseDF = phaseDF[phaseDF['timestamp'] >= datePhaseFrom]
		phaseDF = phaseDF[phaseDF['timestamp'] < datePhaseTo]

		mean = phaseDF['value'].mean()
		stdev = phaseDF['value'].std()
		setItem(item, phase, group, mean, stdev)

		phase += 2
		datePhaseFrom = datePhaseTo
		datePhaseTo += timedelta(days = 21)

	print item
	return item

'''
API - getDailyStats()

@Descrition: 
	- Retrieve the daily environment data from the beginning to end of the experiments
	- Input: home (home id), 
	- Input: envType (data that users want to retrieve, temperature/co2/humidity
	- Output: envDataList

@Arguments
	- home		input, object, includes home id and schedule group
	- envType	input, string, a type of env/energy such as temperature, co2, electricity etc
				see ./common/ibert_common.py for more options
	- start		start date for the analysis
	- end		end date for the analysis
	- dataframe	a full data for 18 weeks for all homes
'''
def getDailyStats(home, envType, envDataList):
	
	means = []

	start = ic.PHASE_ZERO['from']
	end = ic.PHASE_FIVE['to']
	records = dbutil.select(qs.ENV_QUERY_MAP[envType] % (int(home["home"]), start, end))
	
	dateStart = datetime.strptime(start, '%Y-%m-%d %H:%M:%S').date()
	dateEnd = datetime.strptime(end, '%Y-%m-%d %H:%M:%S').date()
	dateCur = dateStart
	
	while (dateCur <= dateEnd):
		# will the number of rooms be considered
		dailyLogs = filter(lambda l: l["timestamp"].date() == dateCur, records)	
		values = [r[qs.ENV_COL_MAP[envType]] for r in dailyLogs]
		
		mean = statistics.mean(values) if len(values) > 0 else float('nan')
		if (len(values) > 1): stdev = statistics.stdev(values)
		elif (len(values) == 1): stdev = 0;
		else: stdev = float('nan')

		envDataList['home'].append(home['home'])
		envDataList['type'].append(envType)		
		envDataList['value'].append(mean)
		envDataList['stdev'].append(stdev)
		envDataList['timestamp'].append(dateCur)
		envDataList['room'].append(float('nan'))		
			
		means.append(mean)
		dateCur += timedelta(days = 1)
		
	return len(means)
