#!/usr/bin/python
# -*- coding: utf-8 

# systems
import sys
import statistics	
import numpy as np

# visualisation
import plotly					
import plotly.graph_objs

from datetime import time, datetime, date, timedelta
from plotly.graph_objs import Scatter, Layout, Box, Bar

# customs
import mysql_util as dbutil	
import query_strings as qs		
import ibert_common as ic

'''
Constants
'''
FILENAME_CONVENTION = "/home/jeehang/navetas_gas_%s_from_%s_to_%s.csv"
ITEM = "%d, "
TIME = "timestamp"
TYPE = "type"

'''
Functions
'''
#
#
#
def setItem(outItem, phase, group, mean, stdev):
	if phase < 0:
		print 'phase: out of range (which should be 0 <= phase < =5'
		return
	
	if phase < 1 or phase > 4:
		outItem[ic.quadrant[phase]] = mean
	else:
		idx = (phase + group - 1) % 4
		qid = idx if idx > 0 else 4
		outItem[ic.quadrant[qid]] = mean

# returns one home's temperature statistics in the quadrant basis
# in : the whole temperature data from phase 0 to phase 5
# in : home id
# 
def getQuadrantStatistics(home, outList):
	item = {'home':'%s', 'before':'%d',
            'after':'%d', 'nvnp':'%d', 'vnp':'%d', 'nvp':'%d', 'vp':'%d'}	

	item['home'] = home['home']

	start = ic.PHASE_ZERO['from']
	end = ic.PHASE_FIVE['to']
	records = dbutil.select(qs.TEMPERATURE % (int(home["home"]), start, end))

	dateStart = datetime.strptime(start, '%Y-%m-%d %H:%M:%S').date()
	dateEnd = datetime.strptime(end, '%Y-%m-%d %H:%M:%S').date()
	datePhaseFrom = dateStart
	datePhaseTo = datePhaseFrom + timedelta(days = 21)
	
	phase = 0
	group = int(home['schedule_group'])

	while (datePhaseTo <= dateEnd):
		phaseList = filter(lambda r : 
						r['timestamp'].date() >= datePhaseFrom and 
						r['timestamp'].date() < datePhaseTo,
						records)
		phaseTempList = [t['temperature'] for t in phaseList]

		mean = statistics.mean(phaseTempList) if len(phaseTempList) > 0 else 5	
		stdev = statistics.stdev(phaseTempList) if len(phaseTempList) > 1 else 0
		setItem(item, phase, group, mean, stdev)

		phase += 1
		datePhaseFrom = datePhaseTo
		datePhaseTo += timedelta(days = 21)

	print item
	outList.append(item)

#
#
#
def visQuadrantStatistics(outList):
	#item = {'home':'%s', 'before':'%d', 'after':'%d', 
	#		'nvnp':'%d', 'vnp':'%d', 'nvp':'%d', 'vp':'%d'}

	traces = []
	axisY = []

	axisX = ["home %s" % item['home'] for item in outList]
	
	'''
	before = [item['before'] for item in outList]
	nvnp = [item['nvnp'] for item in outList]
	vnp = [item['vnp'] for item in outList]	
	nvp = [item['nvp'] for item in outList]
	vp = [item['vp'] for item in outList]
	after = [item['after'] for item in outList]
	'''
	axisY.append([item['before'] for item in outList])
	axisY.append([item['nvnp'] for item in outList])
	axisY.append([item['vnp'] for item in outList])
	axisY.append([item['nvp'] for item in outList])
	axisY.append([item['vp'] for item in outList])
	axisY.append([item['after'] for item in outList])

	for i in range(6):
		trace = Bar(x = axisX,
					y = axisY[i],
					name = ic.quadrantString[i])
		traces.append(trace)

	plotly.offline.plot ({
		"data": traces, 
		"layout": Layout(title="Temperature", barmode = 'stack')
	})

def visOverview(outList):
	traces = []
	annots = []

	axisX = []
	axisY = ic.quadrantString

	for i in range(6):
		mList = filter(lambda d : d > 5, [item[ic.quadrant[i]] for item in outList])
		mean = statistics.mean(mList) if len(mList) > 0 else 0
		axisX.append(mean)
		annots.append(dict(xref = 'x1', yref = 'y1', y = axisY[i], x = mean + 1.5,
							text = str(np.round(mean, decimals = 2)),
							font = dict(family = 'Arial', size = 14, color = 'rgb(55, 128, 191)'),
                            showarrow = False,))

	trace = Bar(x = axisX, y = axisY, orientation = 'h')
	traces.append(trace)
	plotly.offline.plot ({
		"data": traces, 
		"layout": Layout(title = "Temperature Overview", annotations = annots)
	})
