#!/usr/bin/python
# -*- coding: utf-8 

# systems
import sys
import statistics	
import numpy as np

# custom
import ibert_common as ic

# visualisation
import plotly					
import plotly.graph_objs

from datetime import time, datetime, date, timedelta
from plotly.graph_objs import Scatter, Layout, Box, Bar

def visOverview(outList, idxStart, idxEnd):
	traces = []
	annots = []
	idx = idxStart

	axisX = []
	axisY = ic.quadrantString

	while idx <= idxEnd:
		mList = filter(lambda d : d > 0, [item[ic.quadrant[idx]] for item in outList])
		mean = statistics.mean(mList) if len(mList) > 0 else 0
		axisX.append(mean)
		annots.append(dict(xref = 'x1', yref = 'y1', y = axisY[idx], x = mean + 1.5,
						text = str(np.round(mean, decimals = 2)),
						font = dict(family = 'Arial', size = 14, color = 'rgb(55, 128, 191)'),
						showarrow = False,))
		idx += 1

	trace = Bar(x = axisX, y = axisY[idxStart:idxEnd + 1], orientation = 'h')
	traces.append(trace)
	plotly.offline.plot ({
		"data": traces, 
		"layout": Layout(title = "iBert Playing Time Overview", annotations = annots)
	})

def visScatter(listX, listY, idxStart, idxEnd):
	traces = []
	
	while idxStart <= idxEnd:
		axisX = [item[ic.quadrant[idxStart]] for item in listX]
		axisY = [item[ic.quadrant[idxStart]] for item in listY]
		trace = Scatter(x = axisX, y = axisY, 
						mode = 'markers', name = ic.quadrantString[idxStart])
		traces.append(trace)
		idxStart += 1
	
	plotly.offline.plot({
		"data": traces,
		"layout": Layout(title = "temperature against ibert")
	})
	
