#!/usr/bin/python
# -*- coding: utf-8 -*-
'''
@author	JeeHang Lee (jeehanglee@gmail.com)
@date	20160127
@description

This is a script to retrieve the iBert playing time at each home participated in ENLITEN project.
Will support
	1. daily/weekly/in phase statistics
	2. hopefully link to the visualisation
'''

import sys
import getopt
import statistics

import mysql_util as dbutil		# db utils (custom)
import query_strings as qs		# query strings (custom)
import ibert_common as ic
import enliten_visualise as ev

import plotly					# visualisation
import plotly.graph_objs

from plotly.graph_objs import Scatter, Layout, Box, Bar
from datetime import time, datetime, date, timedelta

'''
Constants
'''
ARG_NUM = 4
CMD_HELP = 'iBertPlayingTime.py <option:daily/weekly/phase> \
		<datefrom:YYYY-MM-DD HH:MM:SS> <dateto::YYYY-MM-DD HH:MM:SS>'

FILENAME_CONVENTION = "/home/jeehang/iBert_playing_%s_from_%s_to_%s.csv"

ITEM = "%d, "

TIME = "timestamp"
TYPE = "type"

STATE_APP_CREATED = 100
STATE_APP_STARTED = 101
STATE_APP_RESTARTED = 102
STATE_APP_RESUMED = 103
STATE_APP_PAUSED = 104
STATE_APP_STOPPED = 105
STATE_APP_DESTROYED = 106

'''
Queries
'''
HOMES = "select f.home, f.access_key \
	from feedback_auth f \
	left join tablets t \
	on f.home = t.home \
	where t.deployed is not null \
	order by f.home asc"

'''
Functions
'''

def setItem(outItem, phase, group, mean, stdev):
    if phase < 0:
        print 'phase: out of range (which should be 0 <= phase < =5'
        return

    if phase < 1 or phase > 4:
        outItem[ic.quadrant[phase]] = mean
    else:
        idx = (phase + group - 1) % 4
        qid = idx if idx > 0 else 4
        outItem[ic.quadrant[qid]] = mean

def isInitialState(state, last_state, last_ts):
	res = False

	if (state >= STATE_APP_STARTED
		and last_state is 0
		and last_ts is ''):
		res = True
	else:
		res = False

	return res

def getDailyPlayingTime(daily_logs):
	last_state = 0
	last_ts = ''
	elapsed = 0
	daily_play = 0

	filtered_logs = filter(lambda l: l["type"] > STATE_APP_CREATED 
								and l["type"] < STATE_APP_DESTROYED, daily_logs)

	for log in filtered_logs:
		ts = log["timestamp"]
		state = log["type"]		

		if (isInitialState(state, last_state, last_ts) == True):
			last_ts = ts
			last_state = state
		elif (state == STATE_APP_STOPPED 
			and last_state >= STATE_APP_STARTED
			and last_ts is not ''):
			diff = ts - last_ts
			daily_play += diff.total_seconds()
			last_state = 0			
			last_ts = ''
			#print diff
		else:
			last_state = state

	return daily_play

def getQuadrantStatistics(homes, outList):
    item = {'home':'%s', 'type':"playingtime", 
            'nvnp':'%d', 'vnp':'%d', 'nvp':'%d', 'vp':'%d'}
    
    item['home'] = home['home']
    start = ic.PHASE_ONE['from']
    end = ic.PHASE_FOUR['to']
    records = dbutil.select(qs.PLAY_LOG % (int(home["access_key"]), start, end))
    
    dateStart = datetime.strptime(start, '%Y-%m-%d %H:%M:%S').date()
    dateEnd = datetime.strptime(end, '%Y-%m-%d %H:%M:%S').date()
    datePhaseFrom = dateStart
    datePhaseTo = datePhaseFrom + timedelta(days = 21)
    
    phase = 1
    group = int(home['schedule_group'])
    list_playing_time = []
	#list_playing_time.append(int(home["home"]))
    
    while (datePhaseTo <= dateEnd):
        phaseList = filter(lambda r: r["timestamp"].date() >= datePhaseFrom and 
                           r["timestamp"].date() < datePhaseTo,
                           records)

        dt_cur = datePhaseFrom
        dt_to = datePhaseTo
        while (dt_cur < dt_to):
            daily_logs = filter(lambda l: l["timestamp"].date() == dt_cur, phaseList)
            daily_playing_time = getDailyPlayingTime(daily_logs)
            if daily_playing_time > 0:
                list_playing_time.append(daily_playing_time)
            dt_cur += timedelta(days = 1)
        
        mean = statistics.mean(list_playing_time) / 60 if len(list_playing_time) > 0 else 0
        stdev = statistics.stdev(list_playing_time) / 60 if len(list_playing_time) > 1 else 0
        setItem(item, phase, group, mean, stdev)

        phase += 1
        datePhaseFrom = datePhaseTo
        datePhaseTo += timedelta(days = 21)
        del list_playing_time[:]

    print item
    outList.append(item) 

'''
Program Start
'''

homes = ic.getEnlitenHomes()

phases = []
outList = []

for home in homes:
    getQuadrantStatistics(home, outList)

ev.visOverview(outList, 1, 4)

print "End of Program"


	
