#!/usr/bin/python
# -*- coding: utf-8 -*-
'''
@author	JeeHang Lee (jeehanglee@gmail.com)
@date	20160127
@description

This is a script to retrieve the iBert playing time at each home participated in ENLITEN project.
Will support
	1. daily/weekly/in phase statistics
	2. hopefully link to the visualisation
'''


import sys, getopt				# systems
import statistics
import plotly					# visualisation
import mysql_util as dbutil		# db utils (custom)
import query_strings as qs		# query strings (custom)
import plotly.graph_objs

from datetime import time, datetime, date

from plotly.graph_objs import Scatter, Layout, Box, Bar
import numpy as np

'''
Constants
'''
FILENAME_CONVENTION = "/home/jeehang/navetas_gas_%s_from_%s_to_%s.csv"
ITEM = "%d, "
TIME = "timestamp"
TYPE = "type"

ARG_NUM = 4
CMD_HELP = 'navetas.py <option:daily/weekly/phase> <datefrom:YYYY-MM-DD HH:MM:SS> <dateto::YYYY-MM-DD HH:MM:SS>'

'''
Definition
'''


'''
Functions
'''

# retrieve energy (gas/elec) records from DB for each home
def getEnergyRecords(home, option, datefrom, dateto):
	records = dbutil.select(qs.ENERGY % (int(home["home"]), datefrom, dateto))

	dt_cur = datetime.strptime(datefrom, '%Y-%m-%d %H:%M:%S').date()
	dt_to = datetime.strptime(dateto, '%Y-%m-%d %H:%M:%S').date()
	
	time_from = time(23, 55, 00)
	time_to = time(23, 59, 59)

	daily_consumptions = filter(lambda r: r["timestamp"].time() >= time_from 
								and r["timestamp"].time() < time_to, records)
	print daily_consumptions
	return daily_consumptions

# stats - mean/stdev for each home
def getEnergyStats(homeRecord, option):
	record = {'home':'%d', 'sum':'%d', 'mean':'%d', 'sdev':'%d'}
	record['home'] = homeRecord['home']
	energyRecord = homeRecord['energy']

	if (len(energyRecord) <= 0 or 
		(len(energyRecord) <= 1 and len(energyRecord[0]) <= 0)):
		mean = 0
		stdev = 0
	else:
		gaslist = filter(lambda x: x > 0.0 , [r[option] for r in energyRecord])
		mean = statistics.mean(gaslist) if len(gaslist) > 0 else 0
		stdev = statistics.stdev(gaslist) if len(gaslist) > 1 else 0
	
	record['mean'] = mean
	record['sdev'] = stdev

	print record
	return record

# graph - line graph
def visEnergyStatsLineGraph(stats):	
	xHomes = [h['home'] for h in stats]
	yMeans = [m['mean'] for m in stats]
	plotly.offline.plot ({
		"data": [Scatter(x = xHomes, y = yMeans)], 
		"layout": Layout(title="GAS consumption")
	})

# graph - box plot
def visEnergyStatsBoxPlots(records, option):
	traces = []
	for record in records:
		energy = record['energy']
		trace = Box(
			y = filter(lambda x: x > 0.0 , [r[option] for r in energy]),
			name = "Home %s" % record['home'],
			marker = dict(
				color = 'rgb(8, 81, 156)',
			),
			boxmean = 'sd'
		)
		traces.append(trace)

	#plotly.offline.plot(traces, filename='box')
	plotly.offline.plot ({
		"data": traces, 
		"layout": Layout(title="GAS consumption")
	})

def visEnergyStatsBars(stats):
	traces = []

	xHomes = ["home %d" % h['home'] for h in stats]
	yMeans = [m['mean'] for m in stats]
	sdev = [s['sdev'] for s in stats]

	trace = Bar(
		x = xHomes,
		y = yMeans,
		name = 'Gas Consumption',
		error_y = dict(
			type = 'data',
			array = sdev,
			visible = True
		)
	)
	traces.append(trace)

	plotly.offline.plot ({
		"data": traces, 
		"layout": Layout(title="GAS consumption")
	})

