#!/usr/bin/python
# -*- coding: utf-8 

import sys, getopt, statistics	# systems
import mysql_util as dbutil		# db utils (custom)
import query_strings as qs		# query strings (custom)
import ibert_common as ic

import plotly					# visualisation
import plotly.graph_objs

from datetime import time, datetime, date, timedelta
from plotly.graph_objs import Scatter, Layout, Box, Bar


'''
Constants
'''
FILENAME_CONVENTION = "/home/jeehang/navetas_gas_%s_from_%s_to_%s.csv"
ITEM = "%d, "
TIME = "timestamp"
TYPE = "type"

ARG_NUM = 4
CMD_HELP = 'pi_temperature.py <option:daily/weekly/phase> <datefrom:YYYY-MM-DD HH:MM:SS> <dateto::YYYY-MM-DD HH:MM:SS>'

colors = ['rgba(93, 164, 214, 1)', 'rgba(255, 144, 14, 1)', 'rgba(44, 160, 101, 1)', 'rgba(255, 65, 54, 1)', 'rgba(207, 114, 255, 1)', 'rgba(127, 96, 0, 1)', 'rgba(255, 140, 184, 1)', 'rgba(79, 90, 117, 1)', 'rgba(222, 223, 0, 1)']

'''
Definition
'''


'''
Functions
'''

def getIndividualHomeTemperature(home, option, datefrom, dateto):
	records = dbutil.select(qs.TEMPERATURE % (int(home["home"]), datefrom, dateto))

	daily_list = []
	dt_cur = datetime.strptime(datefrom, '%Y-%m-%d %H:%M:%S').date()
	dt_to = datetime.strptime(dateto, '%Y-%m-%d %H:%M:%S').date()

	while (dt_cur < dt_to):
		stat = {'mean':'%d', 'stdev':'%d', 'timestamp':'%s'}
		
		daily_logs = filter(lambda l: l["timestamp"].date() == dt_cur, records)	
		temp_list = [r['temperature'] for r in daily_logs]
		
		if len(temp_list) > 0:
			mean = statistics.mean(temp_list) if len(temp_list) > 0 else 0
			stdev = statistics.stdev(temp_list) if len(temp_list) > 1 else 0

			stat['mean'] = mean
			stat['stdev'] = stdev
			stat['timestamp'] = dt_cur		
			daily_list.append(stat)
			
		dt_cur += timedelta(days = 1)
		
	print 'home %s: ' % home 
	print daily_list 
	return daily_list

# interface - get temperature for whole homes, on a daily basis
def getDailyTemperature(homes, datefrom, dateto):
	records = []

	if (len(homes) <= 0):
		print 'something wrong'
	
	for home in homes:
		record = {'home':'%d', 'temperature':[]}
		temperature = getIndividualHomeTemperature(home, 'daily', datefrom, dateto)
		
		if len(temperature) > 0:
			record['home'] = home['home']	
			record['temperature'] = temperature
			records.append(record)	
		
	return records

# interface - get temperature for whole homes, on a phase basis
def getPhaseTemperature(homes, datefrom, dateto):
	stats = []

	if (len(homes) <= 0):
		print 'something wrong'
	
	for home in homes:
		stat = {'home':'%d', 'sum':'%d', 'mean':'%d', 'stdev':'%d'}		
		temperature = getIndividualHomeTemperature(home, 'daily', datefrom, dateto)
		avglist = [i['mean'] for i in temperature]
		
		if len(avglist) > 0:
			stat['home'] = home['home']
			stat['mean'] = statistics.mean(avglist) if len(avglist) > 0 else 0	
			stat['stdev'] = statistics.stdev(avglist) if len(avglist) > 1 else 0
			stats.append(stat)	
		
	return stats

# returns one home's temperature statistics in the quadrant basis
# in : the whole temperature data from phase 0 to phase 5
# in : home id
# 
def getQuadrantStatitics(home, outList):
	item = {'home':'%s', 'before':'%d', 'after':'%d', 
			'nvnp':'%d', 'vnp':'%d', 'nvp':'%d', 'vp':'%d'}	

	start = ic.PHASE_ZERO['from']
	end = ic.PHASE_FIVE['to']
	records = dbutil.select(qs.TEMPERATURE % (int(home["home"]), start, end))

	dateStart = datetime.strptime(start, '%Y-%m-%d %H:%M:%S').date()
	dateEnd = datetime.strptime(end, '%Y-%m-%d %H:%M:%S').date()
	datePhaseFrom = dateStart
	datePhaseTo = datePhaseFrom + timedelta(days = 21)
	
	phase = 0
	#group = int(home['schedule_group'])

	while (datePhaseTo <= dateEnd):
		phaseList = filter(lambda r : 
						r['timestamp'].date() >= datePhaseFrom and 
						r['timestamp'].date() < datePhaseTo,
						records)
		mean = statistics.mean([t['temperature'] for t in phaseList])
		print mean		

# graph - line graph
def visRecordsScatter(homes, phases):
	traces = []
	phaseid = 0
	
	touts = []
	touts.append(ic.TEMP_BEFORE_EXP)
	touts.append(ic.TEMP_PHASE_ONE)	
	touts.append(ic.TEMP_PHASE_TWO)

	for record in phases:
		axisX = []
		axisY = []

		for r in record:
			homeid = r['home']
			temperature = r['temperature']	
			cy = [t['mean'] for t in temperature]
			cx = ['home %s' % homeid for i in range(len(cy))]
			axisX.extend(cx)
			axisY.extend(cy)		

		trace = Scatter(
			x = axisX,
			y = axisY,
			mode = 'markers',
			name = 'Phase %d' % phaseid)
		traces.append(trace)
		
		tout = statistics.mean(touts[phaseid])
		trace = Scatter(
			x = ['home %s' % home['home'] for home in homes],
			y = [tout for j in range(len(homes))],
			name = 'Outdoor temperature - phase %d' % phaseid
		)
		traces.append(trace)
		phaseid += 1
	
	plotly.offline.plot ({
		"data": traces, 
		"layout": Layout(title="Temperature in Living Room")
	})


def visRecordsBoxPlot(homes, phases):
	traces = []
	phaseid = 0
	
	touts = []
	touts.append(ic.TEMP_BEFORE_EXP)
	touts.append(ic.TEMP_PHASE_ONE)	
	touts.append(ic.TEMP_PHASE_TWO)

	for record in phases:
		axisX = []
		axisY = []

		for r in record:
			homeid = r['home']
			temperature = r['temperature']	
			cy = [t['mean'] for t in temperature]
			cx = ['home %s' % homeid for i in range(len(cy))]
			axisX.extend(cx)
			axisY.extend(cy)		

		trace = Box(
			x = axisX,
			y = axisY,
			name = 'Phase %d' % phaseid,
			boxpoints=False,
			marker = dict(
				color = colors[phaseid],
			)
		)
		traces.append(trace)
		
		tout = statistics.mean(touts[phaseid])
		trace = Scatter(
			x = ['home %s' % home['home'] for home in homes],
			y = [tout for j in range(len(homes))],
			name = 'T_out - phase %d' % phaseid,
			marker = dict(
				color = colors[phaseid],
			)
		)
		traces.append(trace)
		phaseid += 1

	trace = Scatter(
		x = ['home %s' % home['home'] for home in homes],
		y = [21 for j in range(len(homes))],
		name = 'Reference Temperature',
		marker = dict(
			color = 'rgba(153, 0, 0, 1)',
		)
	)
	traces.append(trace)
	
	plotly.offline.plot ({
		"data": traces, 
		"layout": Layout(title="Temperature in Living Room", boxmode = 'group')
	})


#Program Start

#homes = ic.getEnlitenHomes()
homes = [{'home':'59', 'access_key':'215973'}]
outList = []
getQuadrantStatitics(homes[0], outList)

'''
phases = []

before = getDailyTemperature(homes, ic.BEFORE_EXP['from'], ic.BEFORE_EXP['to'])
phases.append(before)

phase1 = getDailyTemperature(homes, ic.PHASE_ONE['from'], ic.PHASE_ONE['to'])
phases.append(phase1)

phase2 = getDailyTemperature(homes, ic.PHASE_TWO['from'], ic.PHASE_TWO['to'])
phases.append(phase2)

visRecordsBoxPlot(homes, phases)

visRecordsScatter(homes, phases)
'''


# End of Program
